@@ -79,7 +79,10 @@ fs_dirs_create(const char *path) {
 	char curr_path[PATH_MAX + 1];
 	char *end = NULL;
 
-	CHECK(isc_string_copy(curr_path, PATH_MAX, path));
+	/* isc_string_copy has been removed */
+	if (strlcpy(curr_path, path, PATH_MAX) >= PATH_MAX) {
+		CLEANUP_WITH(ISC_R_NOSPACE);
+	}
 
 	for (end = strchr(curr_path, '/');
 	     end != NULL;
