@@ -20,6 +20,14 @@
 #include "settings.h"
 #include "zone_register.h"
 
+#if LIBDNS_VERSION_MAJOR < 1600
+#define cfg_parse_buffer cfg_parse_buffer4
+#define dns_view_flushcache(view, fixup) dns_view_flushcache((view))
+typedef dns_name_t       node_name_t;
+#else
+typedef const dns_name_t node_name_t;
+#endif
+
 const enum_txt_assoc_t forwarder_policy_txts[] = {
 	{ dns_fwdpolicy_none,	"none"	},
 	{ dns_fwdpolicy_first,	"first"	},
@@ -49,7 +57,6 @@ buffer_append_str(void *closure, const char *text, int textlen) {
 	isc_buffer_region(out_buf, &old_space);
 	new_space.length = isc_buffer_length(out_buf) + textlen + 1;
 	new_space.base = isc_mem_get(out_buf->mctx, new_space.length);
-	RUNTIME_CHECK(new_space.base != NULL);
 	isc_buffer_reinit(out_buf, new_space.base, new_space.length);
 	if (old_space.base != NULL)
 		isc_mem_put(out_buf->mctx, old_space.base, old_space.length);
@@ -86,7 +93,6 @@ fwd_list_len(dns_forwarders_t *fwdrs) {
 static isc_result_t
 fwd_list_gen_dummy_config_string(isc_mem_t *mctx, size_t list_len,
 				 isc_buffer_t **dummy_string) {
-	isc_result_t result;
 	const char prefix[] = "{ ";
 	const char suffix[] = "} // dummy string, please ignore";
 	const char fill[] = "127.0.0.1; ";
@@ -98,7 +104,9 @@ fwd_list_gen_dummy_config_string(isc_mem_t *mctx, size_t list_len,
 
 	REQUIRE(dummy_string != NULL && *dummy_string == NULL);
 
-	CHECK(isc_buffer_allocate(mctx, &output, target_size));
+	/* No CHECK here as isc_buffer_allocate is void and can not fail.
+	 *  See bind9: a038f77d92a857bc11750683c9317d70da6fcfdf */
+	isc_buffer_allocate(mctx, &output, target_size);
 	isc_buffer_putstr(output, prefix);
 	for (size_t i = 0; i < list_len; i++)
 		isc_buffer_putstr(output, fill);
@@ -106,11 +114,7 @@ fwd_list_gen_dummy_config_string(isc_mem_t *mctx, size_t list_len,
 	isc_buffer_putuint8(output, '\0');
 	*dummy_string = output;
 
-cleanup:
-	if (result != ISC_R_SUCCESS && output != NULL)
-		isc_buffer_free(&output);
-
-	return result;
+	return ISC_R_SUCCESS;
 }
 
 /**
@@ -122,7 +126,6 @@ cleanup:
 isc_result_t
 fwd_print_bracketed_values_buf(isc_mem_t *mctx, ldap_valuelist_t *values,
 			      isc_buffer_t **string) {
-	isc_result_t result;
 	ldap_value_t *value;
 	const char prefix[] = "{ ";
 	const char suffix[] = "}";
@@ -143,13 +146,14 @@ fwd_print_bracketed_values_buf(isc_mem_t *mctx, ldap_valuelist_t *values,
 	buffer_append_str(&tmp_buf, suffix, 2);
 
 	/* create and copy string from tmp to output buffer */
-	CHECK(isc_buffer_allocate(mctx, string, tmp_buf.used));
+        /* No CHECK here as isc_buffer_allocate is void and can not fail.
+         *  See bind9: a038f77d92a857bc11750683c9317d70da6fcfdf */
+	isc_buffer_allocate(mctx, string, tmp_buf.used);
 	isc_buffer_putmem(*string, isc_buffer_base(&tmp_buf), tmp_buf.used);
 
-cleanup:
 	if (tmp_buf.base != NULL)
 		isc_mem_put(mctx, tmp_buf.base, tmp_buf.length);
-	return result;
+	return ISC_R_SUCCESS;
 }
 
 isc_result_t
@@ -182,8 +186,8 @@ fwd_print_list_buff(isc_mem_t *mctx, dns_forwarders_t *fwdrs,
 	list_len = fwd_list_len(fwdrs);
 	CHECK(fwd_list_gen_dummy_config_string(mctx,
 					       list_len, &dummy_fwdr_buf));
-	CHECK(cfg_parse_buffer(parser, dummy_fwdr_buf,
-			       cfg_type_forwarders, &forwarders_cfg));
+	CHECK(cfg_parse_buffer(parser, dummy_fwdr_buf, NULL, 0,
+			       cfg_type_forwarders, 0, &forwarders_cfg));
 
 	/* Walk through internal representation and cfg representation and copy
 	 * data from the internal one to cfg data structures.*/
@@ -199,7 +203,7 @@ fwd_print_list_buff(isc_mem_t *mctx, dns_forwarders_t *fwdrs,
 	cfg_print(faddresses, buffer_append_str, &tmp_buf);
 
 	/* create and copy string from tmp to output buffer */
-	CHECK(isc_buffer_allocate(mctx, out_buf, tmp_buf.used));
+	isc_buffer_allocate(mctx, out_buf, tmp_buf.used);
 	isc_buffer_putmem(*out_buf, isc_buffer_base(&tmp_buf),
 			  isc_buffer_usedlength(&tmp_buf));
 
@@ -275,7 +279,7 @@ fwd_parse_str(const char *fwdrs_str, isc_mem_t *mctx,
 		addr = *cfg_obj_assockaddr(fwdr_cfg);
 		if (isc_sockaddr_getport(&addr) == 0)
 			isc_sockaddr_setport(&addr, port);
-		CHECKED_MEM_GET_PTR(mctx, fwdr);
+		fwdr = isc_mem_get(mctx, sizeof(*(fwdr)));
 		fwdr->addr = addr;
 		fwdr->dscp = cfg_obj_getdscp(fwdr_cfg);
 		ISC_LINK_INIT(fwdr, link);
@@ -494,7 +498,7 @@ cleanup:
  */
 isc_result_t
 fwd_configure_zone(const settings_set_t *set, ldap_instance_t *inst,
-		   dns_name_t *name)
+		   const dns_name_t *name)
 {
 	isc_result_t result;
 	isc_mem_t *mctx = NULL;
@@ -579,10 +583,11 @@ fwd_configure_zone(const settings_set_t *set, ldap_instance_t *inst,
 	run_exclusive_enter(inst, &lock_state);
 	CHECK(fwd_delete_table(view, name, msg_obj_type, set->name));
 	if (isconfigured == true) {
-		CHECK(dns_fwdtable_addfwd(view->fwdtable, name, &fwdrs,
+		CHECK(dns_fwdtable_addfwd(view->fwdtable,
+					  (node_name_t *) name, &fwdrs,
 					  fwdpolicy));
 	}
-	dns_view_flushcache(view);
+	dns_view_flushcache(view, false);
 	run_exclusive_exit(inst, lock_state);
 	lock_state = ISC_R_IGNORE; /* prevent double-unlock */
 	log_debug(5, "%s %s: forwarder table was updated: %s",
@@ -607,11 +612,11 @@ cleanup:
 }
 
 isc_result_t
-fwd_delete_table(dns_view_t *view, dns_name_t *name,
+fwd_delete_table(dns_view_t *view, const dns_name_t *name,
 		 const char *msg_obj_type, const char *logname) {
 	isc_result_t result;
 
-	result = dns_fwdtable_delete(view->fwdtable, name);
+	result = dns_fwdtable_delete(view->fwdtable, (node_name_t *) name);
 	if (result != ISC_R_SUCCESS && result != ISC_R_NOTFOUND) {
 		log_error_r("%s %s: failed to delete forwarders",
 			    msg_obj_type, logname);
